class PluginPuppetBaselineStoreDiffResultId < ActiveRecord::Migration
  def self.up
    # We index on this column because it is a relatively large amount of data
    # to be comparing without the index, and we need to efficiently find rows
    # that have the same value in this field.
    add_column :baseline_differences, :resource_status_digest, 'CHAR(24)'
    add_index  :baseline_differences, :resource_status_digest

    diffs = BaselineDifference.find(:all, :conditions => "report_resource_id IS NOT NULL")
    if diffs.count > 0 then
      STDOUT.puts "-- calculating baseline difference resource status digests"
      pbar = ProgressBar.new("   ->", diffs.count)

      ids    = diffs.collect {|x| x.report_resource_id }
      # Hashes is indexed by string, because this is the "quickly" version and
      # has not-so-nice usability, in return for better performance.  Which is
      # why we have to 'to_s' the ID below. --daniel 2011-06-21
      hashes = Summarizer.calculate_some_resource_event_hashes_quickly(ids)

      diffs.each do |diff|
        # ...and we don't have a single / proxy primary key, so AR hates us.
        # We do this to get the data into place.  Win. --daniel 2011-06-21
        digest = hashes[diff.report_resource_id.to_s]
        where = [:report_id, :baseline_id, :report_resource_id, :baseline_resource_id].
          map {|key| v = diff.send key ; "#{key} #{v.nil? ? 'IS NULL' : "= #{v}"}" }.
          join(" AND ")
        BaselineDifference.update_all("resource_status_digest = '#{digest}'", where)
        pbar.inc
      end
      pbar.finish
    end
  end

  def self.down
    remove_index  :baseline_differences, :resource_status_digest
    remove_column :baseline_differences, :resource_status_digest
  end
end
